%Start ITALIC BOLD CITALIC CBOLD
%%
<CITALIC>\033R			{ printf("\033-0"); BEGIN 0; }
<CBOLD>\033R			{ printf("\033F");  BEGIN 0; }
<CITALIC>\033B			{ printf("\033-0\033E"); BEGIN CBOLD; }
<CBOLD>\033I			{ printf("\033F\033-1"); BEGIN CITALIC; }
\033B				{ printf("\033E");  BEGIN CBOLD; }
\033I				{ printf("\033-1"); BEGIN CITALIC; }
\033R				{ printf("\033F\033-0"); BEGIN 0; }
<ITALIC>_\010.			{ printf("%c",yytext[2]); }
<ITALIC>_			{ printf(" "); }
<ITALIC>[^_\010](\010[^_\10])+/[^\10]	{
				printf("\033-0\033E%c",*yytext);
				BEGIN BOLD;
				}
<ITALIC>.			{ printf("\033-0%c",*yytext); BEGIN 0; }
<BOLD>" "			ECHO;
<BOLD>_\010.			{
				printf("\033F\033-1%c",yytext[2]);
				BEGIN ITALIC;
				}
<BOLD>[^_\010](\010[^_\10])+/[^\10]	{ printf("%c",*yytext); }
<BOLD>.				{ printf("\033F%c",*yytext); BEGIN 0; }
_\010.				{
				printf("\033-1%c",yytext[2]);
				BEGIN ITALIC;
				}
[^_\010](\010[^_\10])+/[^\10]	{
				printf("\033E%c",*yytext);
				BEGIN BOLD;
				}
.				ECHO;
\n				ECHO;
%%
/**********************************************************************
 * epf -- Epson Print Filter.  This filter takes standard input and looks
 * for pattern used to simulate BOLD and ITALIC (UNDERLINE) on dumb
 * printers.  These patterns are converted to escape sequences that can
 * be used by Epson printers.
 *
 * There are 3 states in this lexical analyzer:
 * 0      -- normal input
 * ITALIC -- currently putting out chars with underlining.
 * BOLD   -- currently putting out chars in boldface.
 *
 * ITALIC is performed on dumb printers with the sequence:
 *	<underscore char><bs><character>
 *      where <bs> is a backspace character
 *
 * BOLD on dumb printers with at least 1 overstrike:
 *	<char><bs><char>...<bs><char>
 *	where the pattern <bs><char> must occur at least once.
 *
 * For efficiency considerations, an underscore encountered while in
 * ITALIC mode is translated to a space.  If true italics are available
 * the underscore would look funny.  If only underlining is available,
 * then the printer's internal underlining (which may be better looking
 * than an underscore) will be used.
 *
 * Similarly, a blank encountered while in BOLD is just output
 * without any state change.
 *
 * The output of the nroff processor with the -T37 option is typical
 * of what epf(1) is expecting.
 *
 * In addition, epf(1) will react to escape sequences as output by
 * cpr(1) when using the -x option.  These sequences are:
 *
 *	<ESC>R	- go to Roman typeface
 *	<ESC>I	- go to Italic typeface
 *	<ESC>B	- go to Bold typeface
 *
 * The output of epf are escape sequences used by the epson printers
 * or a ThinkJet run in alternate (epson emulation) mode.  The
 * output sequences are:
 *
 *	<ESC>-0	- Turn off italic mode
 *	<ESC>-1	- Turn on italic mode
 *	<ESC>F	- Turn off bold mode
 *	<ESC>E	- Turn on bold mode
 *
 * As a general rule, BOLD and ITALIC may not exist at the same time.
 *
 * To compile this beast:
 *	lex epf.l
 *	cc -O -o epf lex.yy.c -ll
 *********************************************************************/
